from __future__ import annotations

import argparse
import json

from together import Models


def add_parser(
    subparsers: argparse._SubParsersAction[argparse.ArgumentParser],
) -> None:
    COMMAND_NAME = "models"
    parser = subparsers.add_parser(COMMAND_NAME)

    child_parsers = parser.add_subparsers(required=True)

    _add_list(child_parsers)
    _add_info(child_parsers)
    _add_instances(child_parsers)
    _add_start(child_parsers)
    _add_stop(child_parsers)


def _add_list(
    parser: argparse._SubParsersAction[argparse.ArgumentParser],
) -> None:
    subparser = parser.add_parser("list")
    subparser.add_argument(
        "--raw",
        help="Raw details of all models",
        default=False,
        action="store_true",
    )
    subparser.set_defaults(func=_run_list)


def _add_info(
    parser: argparse._SubParsersAction[argparse.ArgumentParser],
) -> None:
    subparser = parser.add_parser("info")
    subparser.add_argument(
        "model",
        metavar="MODEL",
        help="Proper Model API string name",
        type=str,
    )
    subparser.add_argument(
        "--raw",
        help="Raw details of all models",
        default=False,
        action="store_true",
    )
    subparser.set_defaults(func=_run_info)


def _add_instances(
    parser: argparse._SubParsersAction[argparse.ArgumentParser],
) -> None:
    subparser = parser.add_parser("instances")
    subparser.add_argument(
        "--raw",
        help="Raw list of instances",
        default=False,
        action="store_true",
    )
    subparser.set_defaults(func=_run_instances)


def _add_start(
    parser: argparse._SubParsersAction[argparse.ArgumentParser],
) -> None:
    subparser = parser.add_parser("start")
    subparser.add_argument(
        "model",
        metavar="MODEL",
        help="Proper Model API string name",
        type=str,
    )
    subparser.set_defaults(func=_run_start)


def _add_stop(
    parser: argparse._SubParsersAction[argparse.ArgumentParser],
) -> None:
    subparser = parser.add_parser("stop")
    subparser.add_argument(
        "model",
        metavar="MODEL",
        help="Proper Model API string name",
        type=str,
    )
    subparser.set_defaults(func=_run_stop)


def _run_list(args: argparse.Namespace) -> None:
    models = Models()
    response = models.list()
    if args.raw:
        print(json.dumps(response, indent=4))
    else:
        model_list = []
        for i in response:
            model_list.append(i["name"])
        print(json.dumps(model_list, indent=4))


def _run_info(args: argparse.Namespace) -> None:
    models = Models()
    response = models.list()

    # list of keys to display by default from models info dict
    visible_keys = [
        "name",
        "display_name",
        "display_type",
        "description",
        "creator_organization",
        "hardware_label",
        "pricing_tier",
        "config",
        "base",
    ]

    for i in response:
        if i["name"] == args.model:
            if args.raw:
                print(json.dumps(i, indent=4))
            else:
                model_info = {key: i[key] for key in visible_keys if key in i}
                print(json.dumps(model_info, indent=4))
            break


def _run_instances(args: argparse.Namespace) -> None:
    models = Models()
    response = models.instances()
    if args.raw:
        print(json.dumps(response, indent=4))
    else:
        started_instances = [key for key in response.keys() if response[key] is True]
        print(json.dumps(started_instances, indent=4))


def _run_start(args: argparse.Namespace) -> None:
    models = Models()
    response = models.start(args.model)
    print(json.dumps(response, indent=4))


def _run_stop(args: argparse.Namespace) -> None:
    models = Models()
    response = models.stop(args.model)
    print(json.dumps(response, indent=4))



min_samples = 100

finetune_model_names = [
    "togethercomputer/LLaMA-2-7B-32K",
    "togethercomputer/llama-2-7b",
    "togethercomputer/llama-2-7b-chat",
    "togethercomputer/llama-2-13b-chat",
    "togethercomputer/llama-2-13b",
    "togethercomputer/RedPajama-INCITE-7B-Base",
    "togethercomputer/RedPajama-INCITE-7B-Chat",
    "togethercomputer/RedPajama-INCITE-7B-Instruct",
    "togethercomputer/RedPajama-INCITE-Base-3B-v1",
    "togethercomputer/RedPajama-INCITE-Chat-3B-v1",
    "togethercomputer/RedPajama-INCITE-Instruct-3B-v1",
    "togethercomputer/Pythia-Chat-Base-7B",
    "togethercomputer/Llama-2-7B-32K-Instruct",
    "togethercomputer/llama-2-70b",
    "togethercomputer/llama-2-70b-chat",
]

# List of models we support and their particular behavior, ie special tokens,
# We will add to and update this list
model_info_dict = {
    "EleutherAI/gpt-j-6b": {},
    "EleutherAI/gpt-neox-20b": {},
    "EleutherAI/pythia-12b-v0": {},
    "EleutherAI/pythia-1b-v0": {},
    "EleutherAI/pythia-2.8b-v0": {},
    "EleutherAI/pythia-6.9b": {},
    "HuggingFaceH4/starchat-alpha": {},
    "NousResearch/Nous-Hermes-13b": {},
    "NousResearch/Nous-Hermes-Llama2-13b": {},
    "NumbersStation/nsql-6B": {},
    "OpenAssistant/oasst-sft-4-pythia-12b-epoch-3.5": {},
    "OpenAssistant/stablelm-7b-sft-v7-epoch-3": {},
    "bigcode/starcoder": {},
    "databricks/dolly-v2-12b": {},
    "databricks/dolly-v2-3b": {},
    "databricks/dolly-v2-7b": {},
    "google/flan-t5-xl": {},
    "huggyllama/llama-13b": {},
    "huggyllama/llama-30b": {},
    "huggyllama/llama-7b": {},
    "lmsys/fastchat-t5-3b-v1.0": {},
    "lmsys/vicuna-13b-v1.3": {},
    "lmsys/vicuna-7b-v1.3": {},
    "prompthero/openjourney": {},
    "runwayml/stable-diffusion-v1-5": {},
    "stabilityai/stable-diffusion-2-1": {},
    "stabilityai/stable-diffusion-xl-base-1.0": {},
    "stabilityai/stablelm-base-alpha-3b": {},
    "stabilityai/stablelm-base-alpha-7b": {},
    "tatsu-lab/alpaca-7b-wdiff": {},
    "timdettmers/guanaco-7b": {},
    "togethercomputer/GPT-JT-6B-v1": {},
    "togethercomputer/GPT-JT-Moderation-6B": {},
    "togethercomputer/GPT-NeoXT-Chat-Base-20B": {},
    "togethercomputer/Koala-7B": {},
    "togethercomputer/LLaMA-2-7B-32K": {"bos_token": "<s>", "eos_token": "</s>"},
    "togethercomputer/Pythia-Chat-Base-7B-v0.16": {"eos_token": "<|endoftext|>"},
    "togethercomputer/Pythia-Chat-Base-7B": {"eos_token": "<|endoftext|>"},
    "togethercomputer/RedPajama-INCITE-7B-Base": {"eos_token": "<|endoftext|>"},
    "togethercomputer/RedPajama-INCITE-7B-Chat": {"eos_token": "<|endoftext|>"},
    "togethercomputer/RedPajama-INCITE-7B-Instruct": {"eos_token": "<|endoftext|>"},
    "togethercomputer/RedPajama-INCITE-Base-3B-v1": {"eos_token": "<|endoftext|>"},
    "togethercomputer/RedPajama-INCITE-Chat-3B-v1": {"eos_token": "<|endoftext|>"},
    "togethercomputer/RedPajama-INCITE-Instruct-3B-v1": {"eos_token": "<|endoftext|>"},
    "togethercomputer/codegen2-16B": {},
    "togethercomputer/codegen2-7B": {},
    "togethercomputer/falcon-40b-instruct": {},
    "togethercomputer/falcon-40b": {},
    "togethercomputer/falcon-7b-instruct": {},
    "togethercomputer/falcon-7b": {},
    "togethercomputer/llama-2-13b-chat": {"bos_token": "<s>", "eos_token": "</s>"},
    "togethercomputer/llama-2-13b": {"bos_token": "<s>", "eos_token": "</s>"},
    "togethercomputer/llama-2-70b-chat": {"bos_token": "<s>", "eos_token": "</s>"},
    "togethercomputer/llama-2-70b": {"bos_token": "<s>", "eos_token": "</s>"},
    "togethercomputer/llama-2-7b-chat": {"bos_token": "<s>", "eos_token": "</s>"},
    "togethercomputer/llama-2-7b": {"bos_token": "<s>", "eos_token": "</s>"},
    "togethercomputer/mpt-30b-chat": {},
    "togethercomputer/mpt-30b": {},
    "togethercomputer/mpt-7b-chat": {},
    "togethercomputer/mpt-7b-instruct": {},
    "togethercomputer/mpt-7b": {},
    "togethercomputer/replit-code-v1-3b": {},
}

jokes_list = [
    {
        "text": "<human>: Why is this joke funny? Q: What does a computer do when it is tired? A: It crashes!\nRosey: Did your computer ever stop working (freeze) or turn itself off when you wanted to use it? If it did, you can say your computer crashed. Computer crashes are frustrating because you cannot get your work done, and sometimes you lose the work you finished most recently, unless you saved it. Thankfully, they do not happen too often."
    },
    {
        "text": "<human>: I'm going to give you a joke. After I give you the joke, please explain it: Q: Where can you go to practice math on New Year’s Eve? A: Times Square!\nRosey: To get better at math, people often people often practice addition (+) or subtraction (-) problems; then they work on multiplication (x) and division (÷), and all the way through calculus and beyond. Practicing multiplication is also known as practicing your times tables. You can say what is seven times nine (7 x 9), or you can can say what is seven multiplied by nine. They mean the same thing, times or multiplied by."
    },
    {
        "text": "<human>: Explain this joke: Q: When does it rain money? A: When there is change in the weather!\nRosey: “The rain in Spain falls mainly on the plain,” according to the song from My Fair Lady. In Peru, they just wish it would rain! But nowhere does it rain money. Rain is water that falls from the sky as part of the evaporation/water cycle. When it is sunny outside, and then the weather changes to cloudy or rainy, we say that there is a change in the weather."
    },
    {
        "text": "<human>: Q: What happens when winter arrives? A: Autumn leaves! Why is this joke funny?\nRosey: In the northern hemisphere winter officially starts on December 21 (winter solstice, astronomical winter), but for many places in the north, it is already cold. Did you know that there is more than one way to mark the start of winter? Meteorologists, people who study the weather, talk about meteorological winter which starts on December 1. When talking about seasons, winter comes after fall; fall is also known as autumn."
    },
    {
        "text": "<human>: Q: Where do roses sleep? A: In a flower bed! Why is this joke funny?\nRosey: According to many surveys, roses are one of the most popular flowers. Although they have thorns, some people say that they are not that hard to grow."
    },
    {
        "text": "<human>: Why is this joke funny? Q: Why did the turkey cross the road? A: So people would think he was a chicken!\nRosey: In many countries throughout the world people celebrate a day of thanks. In the United States, Thanksgiving is celebrated on the 4th Thursday in November; that will be November 24 in 2022. One of the traditional foods for that celebration is turkey."
    },
    {
        "text": "<human>: Why is this joke funny? Q: Why was the Incredible Hulk so good at gardening? A: Because he had a green thumb!\nRosey: Dr. David/Robert Bruce Banner becomes the superhero known as the Hulk, or the Incredible Hulk, when he becomes angry. In that process of becoming the Hulk, his body transforms, becoming larger with lots of muscles, and his skin turns green. When his skin turns green, his whole body turns green, including his thumb."
    },
    {
        "text": "<human>: Why is this joke funny? Q: What’s it like being kissed by a vampire? A: It’s a pain in the neck!\nRosey: Happy Halloween to those of you who celebrate! Many countries around the world celebrate Halloween, or another somewhat-related holiday around this time of year. A Mexican student of mine once commented that Halloween is a lot like Day of the Dead, but several others disagreed. I also disagreed, but can see some connections."
    },
    {
        "text": "<human>: Explain this joke: Q: Why was the jelly so stressed out? A: Because it was spread too thin!\nRosey: Jelly is a food made by boiling fruit juice with sugar until it becomes thick and sort of transparent. Jelly is similar to jam, marmalade, and preserves. This link can explain more about the differences between those different spreads."
    },
    {
        "text": "<human>: Why is this joke funny? Q: Why didn’t the girl trust the ocean? A: Because there was something fishy about it!\nRosey: While there are five named oceans, Arctic, Atlantic, Indian, Pacific and Southern, there is really only one world ocean. I would recommend having much respect for the ocean both because of its power to give life to the earth, and its dangerous power to take life through storms. As you already know the ocean has many fish in it. To trust someone means to have confidence that the person will treat you fairly, be kind, not hurt you."
    },
    {
        "text": "<human>: Why is this joke funny? Q: When is an apple grouchy? A: When it is a crab apple!\nRosey: In the north it is apple-picking season, and time for pumpkin spice everything!"
    },
    {
        "text": "<human>: Why is this joke funny? Q: Why did the kids fire their squirt guns into the air on that windy day? A: They were just shooting the breeze!\nRosey: A squirt gun shoots a stream of water, much like a spray bottle, but in the form of a gun. Many people like to play by squirting water at their friends on a warm day outside. Some people train their pets by squirting water at them (others say that squirting animals does not work or is mean)."
    },
    {
        "text": "<human>: Q: What lies on the bottom of the ocean and twitches? A: A nervous wreck! Why is this joke funny?\nRosey: To twitch means to make small, jerking movements. You may have experienced an eye twitch, or felt a leg twitch when you are falling asleep."
    },
    {
        "text": "<human>: Q: Why are false teeth like stars? A: Because they come out at night! Why is this joke funny?\nRosey: False teeth has to come out at night so peope could clean them."
    },
    {
        "text": "<human>: Q: Why did the farmer win the Nobel Prize? A: Because he was outstanding in his field! Explain this joke.\nRosey: I'll try my best. The Nobel Prize is awarded to people who “have conferred the greatest benefit to mankind.” That is, they have done something really, really important because it is something that is good for a lot of people. Outstanding means exceptionally good. People who win the Nobel Prize are outstanding in their field. In this case, field means the special area that they know a lot about such as literature, science, math, …"
    },
    {
        "text": "<human>: Q: Why are elevator jokes the best? A: Because they work on so many levels! Explain this joke.\nRosey: I'll try my best. What goes up, must come down, says the old adage. (Blood, Sweat and Tears sang about that, too.) The same is true for elevators: Elevators (called a lift in England) go up and down to bring people to different floors in a building. The floors in a building are sometimes called levels. Just like in a video game where you move to the next level, elevators can take you up or down one or more levels."
    },
    {
        "text": "<human>: Q: What kind of candy can you get an an airport? A: Plane M&Ms! Why is this joke funny?\nRosey: Even though there are lots of studies that talk about sugar being bad for you, well, a little bit still tastes good. Every place in the world has candy, and candy can even make for a good joke too!"
    },
    {
        "text": "<human>: Q: How is a good baseball game like good pancakes? A: They both need a good batter! Why is this joke funny?\nRosey: The baseball season in the United States and Canada is half done, so a baseball joke seemed appropriate. In baseball, the person who has the bat and is ready to hit the baseball is called the batter (in the sport of cricket, too). A good batter is able to hit the ball that is pitched, usually at 80 miles per hour or more. That’s fast, and hard to hit! And remember, if a player is able to get a hit 3 out of every 10 pitches, that is considered good."
    },
    {
        "text": "<human>: Q: Why did the composer only compose while lying in bed? A: Because he was composing sheet music! Why is this joke funny?\nRosey: A composer is someone who writes music. Two of the most famous composers are Mozart and Beethoven."
    },
    {
        "text": "<human>: Explain this joke: Q: What did the angry lumberjack say to the tree? A: I have an axe to grind with you!\nRosey: Lumberjacks are people who work cutting down trees so that people can use wood to make things. Many years ago, lumberjacks would use a saw and/or an axe to cut down trees. (Hopefully they planted two new trees for each one cut down!) Before working with an axe, it is important to sharpen the axe. One way to sharpen the axe is to use a grinder."
    },
    {
        "text": "<human>: Why is this joke funny? Q: What do you call a fossil that never does any work? A: A lazy bones!\nRosey: Fossils are parts of a living thing that has been preserved, or left an impression from a very long time ago, perhaps 10,000 years ago or more. The human skull that was recently found in Minnesota was not called a fossil, maybe because it was only 8,000 years old. Bones are a very common type of fossil because they last longer than the soft parts of living things. Because fossils come from living things that have been dead for a very long time, they do not do any work."
    },
    {
        "text": "<human>: Explain this joke: Q: What kind of flowers are great friends? A: Rose buds!\nRosey: Roses are one of the best known flowers. As a rose grows, in the spring you will notice buds. Rose buds are little bumps on the stem of a rose from which the flowers will appear. Yes, roses are beautiful! (Be careful of the thorns, though.)"
    },
    {
        "text": "<human>: Why is this joke funny? Q: Why did the two 4s decide to skip dinner? A: Because they already 8.\nRosey: Told to me by a 6th grader, this one made me laugh! It is rather absurd and it really makes no sense…so let me try to explain it."
    },
    {
        "text": "<human>: Why is this joke funny? Q: What’s the best season for jumping on a trampoline? A: Spring time!\nRosey: The northern hemisphere is about a third of the way through the season of spring. Of course summer will come after spring, and then fall and winter. Spring is a beautiful time of year as the trees bud and bloom, the flowers are reborn, and the days get longer. (Actually, days do not really get longer than 24 hours, but the amount of sunlight during the day increases.)"
    },
    {
        "text": "<human>: Explain this joke: Q: Why are cats good at video games? A: Because they have nine lives!\nRosey: Many video games let you have another chance to keep going if you make a really bad mistake, maybe even getting your character killed. (Other games do not give you another life in the same game, and you have to start over.) I am reminded of the classic video game Pac-Man where you would get three lives. It also reminds me of the pre-video game era when people played pinball, and you had to score the maximum number of points with a limited number of balls."
    },
    {
        "text": "<human>: Explain this joke: Q: Did you hear about the cross-eyed teacher? A: He couldn’t control his pupils!\nRosey: Someone whose eyes do not line up with each other, whose eyes do not look in the same direction, suffers from a medical condition called strabismus. In that situation, people cannot make their eyes look in the same direction. In non-medical terms, people whose eyes look in different directions may be called cross-eyed. The term cross-eyed can be used disrespectfully or can be used as a description. If you look down your nose, you are crossing your eyes. In your eyes, pupils are the center part of the eye, the black circle in the middle of the eyes that lets the light in."
    },
    {
        "text": "<human>: Why is this joke funny? Q: What is the difference between a cat that got xeroxed and a cat that imitates its owner? A: One is a cat copy and the other is a copycat!\nRosey: Sometimes in English, brand names become verbs or nouns. For most people, all facial tissue is Kleenex, all internet searches are Googling, and all bandages are Band-Aids. (The term for this is anthimeria.) So, when you xerox something, you make a copy, a photocopy, because Xerox is a brand name of copiers."
    },
    {
        "text": "<human>: Why is this joke funny? Q: Why did the computer get new glasses? A: To improve his web sight!\nRosey: Every few years, it seems, I need to get new reading glasses that are just a bit stronger. The 20-20 vision of my youth has become more like the impressionist works of Monet. When people want to improver their vision, their eyesight, they often get new glasses."
    },
    {
        "text": "<human>: Q: How did the orange get into the crowded restaurant? A: He squeezed his way in! Why is this joke funny?\nRosey: As COVID cases wane, more and more people are heading out to restaurants. In the case of this joke, even the orange goes into a restaurant."
    },
    {
        "text": "<human>: I'm going to give you a joke. After I give you the joke, please explain it: Q: Can listening to the radio be an addiction? A: Depends on the frequency!\nRosey: Today is World Radio Day, so a radio joke seemed appropriate!"
    },
    {
        "text": "<human>: Q: What is the smartest animal? A: A snake, because no one can pull its leg! Why is this joke funny?\nRosey: There are a lot of smart animals! (My cat was not one of them, but I loved him anyway.) Snakes are not generally included in lists of the smartest animals. In this joke, though, they are the smartest."
    },
    {
        "text": "<human>: I'm going to give you a joke. After I give you the joke, please explain it: Q: Why was the rope so stressed out? A: It was getting itself all tied up in knots!\nRosey: Last week an upset wire, this week a stressed out rope! Such is the pandemic life."
    },
    {
        "text": "<human>: Q: Why was the metal wire so upset? A: It was getting bent out of shape over nothing! Explain this joke.\nRosey: I'll try my best. Metal wire comes in all sorts of sizes, measured by how thick it is. Electrical wire also comes in different thicknesses. Metal wire can be bent into many shapes so it is useful for art, for wiring a house, making motors, … When you bend a wire, you can make it into almost any shape"
    },
    {
        "text": "<human>: I'm going to give you a joke. After I give you the joke, please explain it: Q: What happened to the mouse when he fell into the bathtub? A: He came out squeaky clean!\nRosey: Just as a cow says “moo” and a pig says “oink,” mice squeak. If you are interested in hearing an hour of mice squeaking you can listen here (although I am not sure why you would want to)."
    },
    {
        "text": "<human>: Q: What did the orange say to the banana when they were looking for the apple? A: Keep your eyes peeled! Explain this joke.\nRosey: I'll try my best. Whenever you eat a banana you have to peel it, that it, take off the yellow skin. There are many ways to peel a banana. To keep your eyes peeled means to keep your eyes open and watch carefully for something. I always keep my eyes peeled for new jokes that need explaining."
    },
    {
        "text": "<human>: Q: Why did the past year seem to go by in a blur? A: Your resolution must have been too low! Explain this joke.\nRosey: I'll try my best. A new year’s resolution is a decision to do something (or stop doing something) so that your life will be better in the new year. Many people make a resolution to exercise more or read more; many people do not keep their new year’s resolutions."
    },
    {
        "text": "<human>: Explain this joke: Q: Why didn’t the aliens on the UFO land on Christmas morning? A: They were selfish, and did not want to share their presence!\nRosey: UFOs are unidentified flying objects, often seen as flying saucers. Flying saucers (if they exist) come from other galaxies and may have alien life on board. Wouldn’t it be fun to have proof of life from other parts of the universe! While there are many stories and pictures of UFOs, the existence of space aliens has never been proven… yet. They have never made their presence known to us."
    },
    {
        "text": "<human>: Q: What is the nosiest game you can play? A: Racket-ball! Explain this joke.\nRosey: I'll try my best. Some people like some noise in their lives while others prefer a quiet environment. Some of us are in between–sometimes I like a little music or background noise but mostly I prefer quiet conversation."
    },
    {
        "text": "<human>: Why is this joke funny? Q: Why did the Christmas tree go to the barber? A: Because it needed to be trimmed!\nRosey: December is the season when many people prepare to celebrate Christmas. For those who celebrate, buying and decorating a Christmas tree is often very important. Decorating the tree is also called trimming the tree."
    },
    {
        "text": "<human>: Q: Why did the ice cream cone become a news reporter? A: Because he wanted to get the scoop! Why is this joke funny?\nRosey: Do you watch the evening news? Listen to news on the radio or a podcast? Perhaps you read a newspaper? News reporters are always looking to provide their watchers, listeners, and readers with information about what is happening. Sometimes they are able to report on a big, important story, or one that no one has heard yet. That big, important story, unheard story is called a scoop. If a reporter gets the scoop, she or he gets an important story."
    },
    {
        "text": "<human>: Why is this joke funny? Q: Why don’t scientists trust the atoms they work with? A: Because they make up everything!\nRosey: Everything is made of atoms. Yes, those atoms, the ones that are found on the periodic table of the elements. It is still hard for me to believe that those little atoms make up the air we breathe, the trees that grow, the metal in my fork and knife. In this case, to make up something means that it is a part that makes up the whole of something."
    },
    {
        "text": "<human>: Why is this joke funny? Q: Why don’t mummies take vacations from their work? A: Because they are afraid to unwind!\nRosey: Most people enjoy taking a few days off from school or work to rest and relax, to unwind. In this sense, to unwind means to relax after work, after school, or after a stressful time."
    },
    {
        "text": "<human>: I'm going to give you a joke. After I give you the joke, please explain it: Q: Why can’t you play hide and seek with mountains? A: Because they are always peaking!\nRosey: Hide and seek is a game that children play. In this game, one or more child finds a place to hide while another child closes his or her eyes and counts to 10. When the person counting gets to 10, that person goes to look for the ones who are hiding The person who is counting has to close his or her eyes in order not to see where the people are going to hide."
    },
    {
        "text": "<human>: Q: What is the best way to talk to a giant? A: Use big words! Why is this joke funny?\nRosey: Giants are larger than life beings that can be good or evil, or a bit of both. Giants, though, are generally considered to be the antagonists in a story, the bad guys."
    },
    {
        "text": "<human>: Explain this joke: Q: Why did the thief wash his clothes after the robbery? A: He wanted to make a clean getaway!\nRosey: A thief is a robber, someone who steals things. Washing your clothes, well, you already know, will make your clothes clean. A robbery is the moment when something was stolen. If you want to get technical, the law in the United States differentiates between burglary, robbery, and theft, but people do not usually make that difference when they speak."
    },
    {
        "text": "<human>: I'm going to give you a joke. After I give you the joke, please explain it: Q: Why didn’t the zombie go to work? A: Because he felt rotten!\nRosey: When you are sick, it is very important that you don’t go to work or school, especially now with COVID still making people sick. If you feel really sick you might say that you feel rotten; to feel rotten is an idiom meaning to feel sick."
    },
    {
        "text": "<human>: Q: Why do birds fly south for the winter? A: Because it is too far to walk! Explain this joke.\nRosey: I'll try my best. Where I grew up, in Minnesota, it was very common to see birds fly south for the winter. When all of the lakes freeze, the trees lose their leaves, and ground is covered with snow, most birds cannot find food. It’s also just too cold for them to survive. So, they migrate to an area where they will be able to survive."
    },
    {
        "text": "<human>: Explain this joke: Q: What time is it when your clock strikes 13? A: Time to get a new clock!\nRosey: When a clock “strikes the hour” it means that it is exactly on the hour, 8:00 or 11:00 for example (not 8:17 or 11:36) and the bells will chime. Here is Big Ben striking the hour."
    },
    {
        "text": "<human>: Q: Why did the dentist and the manicurist decide not to be friends? A: Because they were always fighting tooth and nail! Why is this joke funny?\nRosey: A dentist is a doctor that improves the health of your teeth. A manicurist is a specialist who improves the health and appearance of your fingernails."
    },
    {
        "text": "<human>: Explain this joke: Q: Is collecting shells on the beach easy to learn? A: Yes, you’ll pick it up right away!\nRosey: Picking up shells on a beach is easy because, if the beach has shells, you just bend over and lift them up with your hands."
    },
    {
        "text": "<human>: Q: Why was the pirate sad when his parrot left? A: It gave him the cold shoulder! Explain this joke.\nRosey: I'll try my best. Yes, the last parrot joke for a while."
    },
    {
        "text": "<human>: Explain this joke: Q: What is bright orange and sounds like a parrot? A: A carrot!\nRosey: Yes! A second parrot joke!"
    },
    {
        "text": "<human>: Q: What creature is smarter than a talking parrot? A: A spelling bee! Why is this joke funny?\nRosey: And we’re back!"
    },
    {
        "text": "<human>: I'm going to give you a joke. After I give you the joke, please explain it: Q: Where do ghosts like to go sailing? A: Lake Eerie!\nRosey: If you ever get the chance to go sailing, I would highly recommend it. It is a beautiful experience to let the wind move your boat without using a motor."
    },
    {
        "text": "<human>: I'm going to give you a joke. After I give you the joke, please explain it: Q: Why didn’t the girl trust the trees near her home? A: They seemed a little shady!\nRosey: Shade is the darker, cooler area under a tree when the sun is shining on the tree. If you want to get out of the sun, you might stand under a tree in the shade. You might stand in the shady area."
    },
    {
        "text": "<human>: Explain this joke: Q: Why couldn’t the pony talk? A: He was a little horse!\nRosey: For most people, a pony is a small horse. There is actually a difference between a horse and a pony. In common conversation, though, most people do not make the distinction. A baby horse is called a foal, colt, or filly depending on age and gender."
    },
    {
        "text": "<human>: Explain this joke: Q: Why aren’t dogs good dancers? A: They have two left feet!\nRosey: Dogs have four feet called paws: two front feet and two back feet. You could also say that they have two right feet and two left feet. Sounds simple."
    },
    {
        "text": "<human>: Q: Why did the baker go to the bank? A: He need some dough! Explain this joke.\nRosey: I'll try my best. There is nothing better than fresh baked bread! The smell is amazing and, if you can eat bread, the taste is divine."
    },
    {
        "text": "<human>: Explain this joke: Q: Did you hear about the new corduroy pillow? A: No, but I bet they’re making headlines!\nRosey: I love my pillow! I have had it for, well, too long, and I should probably get a new one. Pillows are those soft cushions that you put under your head when you sleep. Pillows can also be found on sofas. Many people use pillows to support their back when they are sitting."
    },
    {
        "text": "<human>: I'm going to give you a joke. After I give you the joke, please explain it: Q: How does a cucumber become a pickle? A: It goes through a jarring experience!\nRosey: Do you love pickles? I do!"
    },
    {
        "text": "<human>: Why is this joke funny? Q: What do you call a carrot that insults a rabbit? A: A fresh vegetable!\nRosey: It is springtime in the northern hemisphere and it is time to plant the vegetable garden. That way, you can eat fresh vegetables when they are ripe and ready to be picked. Fresh vegetables are vegetables that have been recently harvested, recently picked."
    },
    {
        "text": "<human>: Why is this joke funny? Q: What does a dog do that a person steps into? A: Pants!\nRosey: Thank you to a friend from India for sending another joke about pants!"
    },
    {
        "text": "<human>: Q: Why did the golfer wear two pairs of pants? A: In case he got a hole in one! Why is this joke funny?\nRosey: I have never been golfing but this joke made me laugh."
    },
    {
        "text": "<human>: Q: What did the big flower say to the little flower? A: Hi, bud! Explain this joke.\nRosey: I'll try my best. Spring is in the air in the northern hemisphere and gardens are blooming!"
    },
    {
        "text": "<human>: Why is this joke funny? Q: Why did Humpty Dumpty have a great fall? A: He wanted to make up for a lousy summer!\nRosey: In my neck of the woods, summer has turned to fall."
    },
    {
        "text": "<human>: Q: Why did the atoms cross the road? A: It was time to split! Explain this joke.\nRosey: I'll try my best. Atoms are the building blocks of matter, of all things. The element chart lists all the different types of atoms."
    },
    {
        "text": "<human>: Why is this joke funny? Q: What is Irish and sits outside? A: Patio Furniture!\nRosey: March 17th is St. Patrick’s Day. Being of Irish heritage, I thought this joke appropriate… and it made me laugh."
    },
    {
        "text": "<human>: Explain this joke: Q: Why was Cinderella so lousy at baseball? A: Because she ran away from the ball and had a pumpkin for a coach!\nRosey: In the United States, baseball spring training has started in Florida and Arizona so how about a baseball joke?"
    },
    {
        "text": "<human>: Why is this joke funny? Q: While in their tank, what did one fish say to the other fish? A: Do you know how to drive this thing?!\nRosey: If you have a fish at home, it is in a tank: a fish tank (also called an aquarium). Normally when you hear about a tank and fish, you think of a fish tank. A fish tank is a glass box that holds water and fish."
    },
    {
        "text": "<human>: Q: What did the right eye say to the left eye? A: Hey buddy, just between you and me, something smells! Why is this joke funny?\nRosey: The phrase “something smells” usually means that something is giving off, emitting, a bad smell. In this joke, “something smells” refers to the nose and the action of smelling, of noticing that there is a odor or aroma that the nose can detect."
    },
    {
        "text": "<human>: Why is this joke funny? Q: Why did the scientist install a knocker on his front door? A: Because he wanted to win the no-bell prize!\nRosey: A knocker is that metal piece on the outside of a door that you use to make noise so that people know that you are there. An alternative to using a door knocker is to use a doorbell. A doorbell is a button, typically found next to the door, that you push to make noise inside the house so that people know that you are there."
    },
    {
        "text": "<human>: Explain this joke: Q: What did the fish say when he swam into the concrete wall? A: Dam!\nRosey: As a language learner, it is important to know the swear words, the bad words, the words you probably shouldn’t say."
    },
    {
        "text": "<human>: Q: Why can’t a woman ask her brother for help? A: Because he can’t be a brother and assist her too! Explain this joke.\nRosey: I'll try my best. Of course a woman can ask her brother for help! Just not in this joke."
    },
    {
        "text": "<human>: Q: How do you keep a bagel from getting away? A: Put lox on it! Explain this joke.\nRosey: I'll try my best. Bagels are a type of bread that look like a donut. They are round and have a hole in the middle, but they are not sweet. To eat a bagel, you usually cut it in half and put something on it like cream cheese."
    },
    {
        "text": "<human>: Q: What do you call a bear with no ears? A: B! Explain this joke.\nRosey: I'll try my best. "
    },
    {
        "text": "<human>: Why is this joke funny? Q: What did the mouse say to the keyboard? A: You are really my type!\nRosey: Computer humor!"
    },
    {
        "text": "<human>: Explain this joke: Q: If Mr. and Mrs. Bigger had a baby, who would be the biggest of all three of them? A: The baby because the baby is a little Bigger!\nRosey: Happy New Year! A baby joke for the new year seemed appropriate."
    },
    {
        "text": "<human>: Q: Where does satisfaction come from? A: A satisfactory! Why is this joke funny?\nRosey: Satisfaction is that feeling you get when things work out. I think of it as a feeling of calm joy because life is good, if only for a moment. There are other meanings of satisfaction, and the Rolling Stones sang about not getting any.)"
    },
    {
        "text": "<human>: Explain this joke: Q: What do you call Santa’s helpers? A: Subordinate Clauses!\nRosey: Merry Christmas!"
    },
    {
        "text": "<human>: Q: Where did the skunk sit when it went to church? A: In its own pew! Explain this joke.\nRosey: I'll try my best. Skunks are those small, squirrel-like animals that are colored mostly black with a white stripe down the back. Perhaps you remember that famous cartoon skunk, Pepe Le Pew? Skunks are known for protecting themselves by spraying a very bad smell."
    },
    {
        "text": "<human>: Q: Did you hear about the two antennas who got married? A: The wedding was awful but the reception was great! Explain this joke.\nRosey: I'll try my best. Antennas are used to receive and send electromagnetic waves so we can communicate (like with radios and TVs). Some antennas are a simple piece of wire while others are huge dishes."
    },
    {
        "text": "<human>: I'm going to give you a joke. After I give you the joke, please explain it: Q: How do you get down from an elephant? A: You don’t get down from an elephant, you get down from a duck!\nRosey: Yes–another duck joke!"
    },
    {
        "text": "<human>: Q: What did the turkey say to the hunter? A: Quack, quack! Explain this joke.\nRosey: I'll try my best. Happy Thanksgiving Day to readers who are celebrating Thanksgiving today. I hope you are safe, and healthy. Thank you for reading Explain the Joke!"
    },
    {
        "text": "<human>: Q: What happened when the ship ran aground while transporting red paint and black paint? A: The crew was marooned! Explain this joke.\nRosey: I'll try my best. A bit of transportation humor this week. To run aground means that the ship was not floating, but the bottom of the ship was touching the ocean floor (the ground under the water). If a ship runs aground it is like a car hitting something and a big mess can occur. The ship in the joke was carrying red paint and black paint, so all all of the paint probably spilled and mixed together creating the color maroon."
    },
    {
        "text": "<human>: Q: How do you get a squirrel to like you? A: Act like a nut! Explain this joke.\nRosey: I'll try my best. In English, the word nut can refer to foods such as pistachios, cashews, and walnuts. Squirrels love to eat nuts, especially acorns if there are oak trees nearby."
    },
    {
        "text": "<human>: I'm going to give you a joke. After I give you the joke, please explain it: Q: Why do skeletons stay so calm? A: Because nothing gets under their skin!\nRosey: Happy Halloween!"
    },
    {
        "text": "<human>: I'm going to give you a joke. After I give you the joke, please explain it: Q: How does the moon cut his hair? A: Eclipse it!\nRosey: A lunar eclipse happens during a full moon when the earth moves between the sun and the moon so that the full moon can no longer be seen. The moon, of course does not have hair but that is part of what makes this joke funny."
    },
    {
        "text": "<human>: I'm going to give you a joke. After I give you the joke, please explain it: Q: What do you get when you combine a porcupine and a turtle? A: A slowpoke!\nRosey: A porcupine is an animal that has needles which are called quills. The quills are sharp on the end and will hurt if they touch you, if they poke you (like when you get a shot in the arm)."
    },
    {
        "text": "<human>: Q: What’s worse than finding a worm in your apple? A: Finding half a worm! Why is this joke funny?\nRosey: Imagine biting into an apple and finding a worm. Yes, that is a very unpleasant thought because your apple is most likely not in very good condition."
    },
    {
        "text": "<human>: Explain this joke: Q: Why was the baby strawberry crying? A: Because her parents were in a jam!\nRosey: Jam is a type of food that is typically made of fruit. Jam is usually spread on another food like bread or toast (like the way butter is spread on bread or toast). If you use strawberries to make jam, you have made strawberry jam. Mmmm! I like it."
    },
    {
        "text": "<human>: I'm going to give you a joke. After I give you the joke, please explain it: Q: What did the baby corn say to the mama corn? A: Where is pop corn?\nRosey: There are many types of families. Some families, like the one in this joke, are made of a mom, dad, and baby. Mom can be called mama; dad can be called pop."
    },
    {
        "text": "<human>: Q: What runs but never walks, murmurs, but never talks, has a bed but never sleeps, and has a mouth but never eats? A: A river! Why is this joke funny?\nRosey: Thanks the Instagram account of Glacier National Park for this week’s joke."
    },
    {
        "text": "<human>: Q: When does a joke turn into a “dad” joke? A: When the answer is a parent! Why is this joke funny?\nRosey: There is a category of jokes called dad jokes. These are jokes that have really bad answers that will make you groan because they are stupid/silly. Puns fit nicely into this category. I rather like theses jokes. By the way, the answer to a joke is also called a punchline."
    },
    {
        "text": "<human>: Why is this joke funny? Q: What is worse than raining cats and dogs? A: Hailing taxis!\nRosey: Precipitation is water in any form that falls from the sky. This includes rain, snow, sleet and hail. Hail is frozen rain, ice, that falls; it can be very small or quite large."
    },
    {
        "text": "<human>: I'm going to give you a joke. After I give you the joke, please explain it: Q: How do you stop an astronaut’s baby from crying? A: You rocket!\nRosey: Astronauts are scientists who explore and study space. To get into space they blast off in a rocket. In this case, ‘rocket’ is a noun, a thing."
    },
    {
        "text": "<human>: Q: What did the Dalmatian say after eating lunch? A: Wow, that really hit the spot! Explain this joke.\nRosey: I'll try my best. If you have seen the movie 101 Dalmatians, you know that Dalmatians are a breed of dog known for their spots. While I like dogs, having 101 of them seems like a lot!"
    },
    {
        "text": "<human>: Q: Why did the child cross the playground? A: To get to the other slide! Explain this joke.\nRosey: I'll try my best. Back in 2013, the first joke that I explained was, perhaps, the most famous of all jokes–Why did the chicken cross the road? This joke is funny because it plays with that joke."
    },
    {
        "text": "<human>: Explain this joke: Q: Why do you always find something in the last place you look? A: Because when you find it, you stop looking!\nRosey: Perhaps this is more of a logic puzzle than a joke–but it is true! Often times we search long and hard for a missing thing and then find it in the last place we look. Is there a psychological reason it took us so long to find something? I do not know the answer to that. I do know that this joke makes me laugh."
    },
    {
        "text": "<human>: Explain this joke: Q: What music frightens balloons? A: Pop music!\nRosey: When balloons explode, we say they pop. If you stick a pin in a balloon, it pops. If balloons had feelings, I am pretty sure that they would be scared of things that make the pop."
    },
    {
        "text": "<human>: Explain this joke: Q: What race is never run? A: A swimming race!\nRosey: Thank you to a swimming friend who sent me this joke."
    },
]



from __future__ import annotations

import argparse
import json

from together import Files, extract_time


def add_parser(subparsers: argparse._SubParsersAction[argparse.ArgumentParser]) -> None:
    COMMAND_NAME = "files"
    parser = subparsers.add_parser(COMMAND_NAME)

    child_parsers = parser.add_subparsers(required=True)

    _add_list(child_parsers)
    _add_check(child_parsers)
    _add_upload(child_parsers)
    _add_delete(child_parsers)
    _add_retrieve(child_parsers)
    _add_retrieve_content(child_parsers)


def _add_list(parser: argparse._SubParsersAction[argparse.ArgumentParser]) -> None:
    subparser = parser.add_parser("list")
    subparser.set_defaults(func=_run_list)


def _add_check(parser: argparse._SubParsersAction[argparse.ArgumentParser]) -> None:
    subparser = parser.add_parser("check")
    subparser.add_argument(
        "file",
        metavar="FILENAME",
        help="Local file to upload",
        type=str,
    )
    subparser.add_argument(
        "--model",
        "-m",
        default=None,
        metavar="MODELNAME",
        help="check data for this model's special tokens",
        type=str,
        required=False,
    )
    subparser.set_defaults(func=_run_check)


def _add_upload(parser: argparse._SubParsersAction[argparse.ArgumentParser]) -> None:
    subparser = parser.add_parser("upload")
    subparser.add_argument(
        "file",
        metavar="FILENAME",
        help="Local file to upload",
        type=str,
    )
    subparser.add_argument(
        "--no-check",
        default=False,
        action="store_true",
        help="Indicates whether to disable checking",
    )
    subparser.add_argument(
        "--model",
        "-m",
        default=None,
        metavar="MODELNAME",
        help="check data for this model's special tokens",
        type=str,
        required=False,
    )
    subparser.set_defaults(func=_run_upload)


def _add_delete(parser: argparse._SubParsersAction[argparse.ArgumentParser]) -> None:
    subparser = parser.add_parser("delete")
    subparser.add_argument(
        "file_id",
        metavar="FILE-ID",
        help="File ID of remote file",
        type=str,
    )
    subparser.set_defaults(func=_run_delete)


def _add_retrieve(parser: argparse._SubParsersAction[argparse.ArgumentParser]) -> None:
    subparser = parser.add_parser("retrieve")
    subparser.add_argument(
        "file_id",
        metavar="FILE-ID",
        help="File ID of remote file",
        type=str,
    )
    subparser.set_defaults(func=_run_retrieve)


def _add_retrieve_content(
    parser: argparse._SubParsersAction[argparse.ArgumentParser],
) -> None:
    subparser = parser.add_parser("retrieve-content")
    subparser.add_argument(
        "file_id",
        metavar="FILE-ID",
        help="File ID of remote file",
        type=str,
    )
    subparser.add_argument(
        "--output",
        "-o",
        default=None,
        metavar="OUTPUT_FILE",
        help="Optional output filename. Defaults to remote filename.",
        type=str,
        required=False,
    )

    subparser.set_defaults(func=_run_retrieve_content)


def _run_list(args: argparse.Namespace) -> None:
    files = Files()
    response = files.list()
    response["data"].sort(key=extract_time)
    print(json.dumps(response, indent=4))


def _run_check(args: argparse.Namespace) -> None:
    files = Files()
    response = files.check(args.file, args.model)
    print(json.dumps(response, indent=4))


def _run_upload(args: argparse.Namespace) -> None:
    files = Files()
    response = files.upload(file=args.file, check=not args.no_check, model=args.model)
    print(json.dumps(response, indent=4))


def _run_delete(args: argparse.Namespace) -> None:
    files = Files()
    response = files.delete(args.file_id)
    print(json.dumps(response, indent=4))


def _run_retrieve(args: argparse.Namespace) -> None:
    files = Files()
    response = files.retrieve(args.file_id)
    print(json.dumps(response, indent=4))


def _run_retrieve_content(args: argparse.Namespace) -> None:
    files = Files()
    output = files.retrieve_content(args.file_id, args.output)
    print(output)



from __future__ import annotations

import argparse
import base64
import json
import logging
import sys
from typing import Any, Dict

import together
from together import Image, get_logger


def add_parser(subparsers: argparse._SubParsersAction[argparse.ArgumentParser]) -> None:
    COMMAND_NAME = "image"
    subparser = subparsers.add_parser(COMMAND_NAME)

    subparser.add_argument(
        "prompt",
        metavar="PROMPT",
        default=None,
        type=str,
        help="A string providing context for the model to complete.",
    )

    subparser.add_argument(
        "--model",
        "-m",
        default=together.default_image_model,
        type=str,
        help=f"The name of the model to query. Default={together.default_image_model}",
    )

    subparser.add_argument(
        "--negative-prompt",
        "-np",
        default=None,
        type=str,
        help="Negative prompt",
    )

    subparser.add_argument(
        "--height",
        default=256,
        type=int,
        help="Pixel height for generated image results",
    )
    subparser.add_argument(
        "--width",
        default=256,
        type=int,
        help="Pixel width for generated image results",
    )
    subparser.add_argument(
        "--steps",
        default=20,
        type=int,
        help="Number of steps",
    )
    subparser.add_argument(
        "--seed",
        default=42,
        type=int,
        help="Seed for image generation",
    )
    subparser.add_argument(
        "--results",
        "-r",
        default=1,
        type=int,
        help="Number of image results to return",
    )
    subparser.add_argument(
        "--output-prefix",
        "-o",
        default="image",
        type=str,
        metavar="PREFIX",
        help="Prefix for the file names the output images will be saved to. An image number will be appended to this name. Default=image",
    )
    subparser.add_argument(
        "--raw",
        default=False,
        action="store_true",
        help="Indicates whether to output raw image to CLI. Enabling this option does not save the image to disk.",
    )

    subparser.set_defaults(func=_run_complete)


def _save_image(
    args: argparse.Namespace, response: Dict[str, Any], logger: logging.Logger
) -> None:
    if args.raw:
        print(json.dumps(response, indent=4))
        sys.exit()

    if "output" in response.keys():
        try:
            images = response["output"]["choices"]

            for i in range(len(images)):
                with open(f"{args.output_prefix}-{i}.png", "wb") as f:
                    f.write(base64.b64decode(images[i]["image_base64"]))
                print(f"Image saved to {args.output_prefix}-{i}.png")
        except Exception as e:  # This is the correct syntax
            logger.critical(f"Error raised: {e}")
            raise together.ResponseError(e)

    elif "error" in response.keys():
        if response["error"] == "Returned error: no instance":
            logger.critical(
                f"No running instances for {args.model}. You can start an instance by navigating to the Together Playground at api.together.xyz"
            )
            raise together.InstanceError(model=args.model)
        else:
            logger.critical(f"Error raised: {response['error']}")

    else:
        logger.critical("Unknown response received.")
        raise together.ResponseError("Unknown response received.")


def _run_complete(args: argparse.Namespace) -> None:
    logger = get_logger(__name__, log_level=args.log)

    complete = Image()

    response = complete.create(
        prompt=args.prompt,
        model=args.model,
        steps=args.steps,
        seed=args.seed,
        results=args.results,
        height=args.height,
        width=args.width,
        negative_prompt=args.negative_prompt,
    )

    _save_image(args, response, logger)



#! python
import argparse

import together
from together import get_logger
from together.commands import chat, complete, files, finetune, image, models


def main() -> None:
    parser = argparse.ArgumentParser(
        description="Together CLI",
        prog="together",
    )

    parser.add_argument(
        "-V",
        "--version",
        action="version",
        version="%(prog)s " + together.version,
    )

    parser.add_argument(
        "--endpoint",
        "-e",
        help="[Optional] Together API Endpoint URL",
        type=str,
        required=False,
        default=None,
    )

    parser.add_argument(
        "--log",
        default=together.log_level,
        choices=["CRITICAL", "ERROR", "WARNING", "INFO", "DEBUG"],
        type=str,
        help="Set logging level. Defaults to WARNING. DEBUG will show all logs.",
        required=False,
    )

    subparser = parser.add_subparsers(dest="base")

    models.add_parser(subparser)
    chat.add_parser(subparser)
    complete.add_parser(subparser)
    image.add_parser(subparser)
    files.add_parser(subparser)
    finetune.add_parser(subparser)

    args = parser.parse_args()

    # Setup logging
    try:
        get_logger(__name__, log_level=args.log)
    except Exception:
        get_logger(__name__, log_level=together.log_level)

    together.log_level = args.log

    try:
        args.func(args)
    except AttributeError as e:
        # print error, but ignore if `together` is run.
        if str(e) != "'Namespace' object has no attribute 'func'":
            raise together.AttributeError(e)
        parser.print_help()


if __name__ == "__main__":
    main()



import urllib.parse
from typing import Any, Dict, List

import requests

import together
from together import get_logger, verify_api_key


logger = get_logger(str(__name__), log_level=together.log_level)


class Models:
    def __init__(
        self,
    ) -> None:
        verify_api_key(logger)

    @classmethod
    def list(self) -> List[Any]:
        model_url = urllib.parse.urljoin(together.api_base, "models/info?=")
        headers = {
            "Authorization": f"Bearer {together.api_key}",
            "User-Agent": together.user_agent,
        }
        try:
            response = requests.get(
                model_url,
                headers=headers,
            )
            response.raise_for_status()
        except requests.exceptions.RequestException as e:
            logger.critical(f"Response error raised: {e}")
            raise together.ResponseError(e)

        try:
            response_list = list(response.json())
        except Exception as e:
            logger.critical(
                f"JSON Error raised: {e}\nResponse status code = {response.status_code}"
            )
            raise together.JSONError(e, http_status=response.status_code)

        return response_list

    @classmethod
    def instances(self) -> Dict[str, bool]:
        headers = {
            "Authorization": f"Bearer {together.api_key}",
            "accept": "application/json",
        }
        try:
            response = requests.get(
                together.api_base_instances,
                headers=headers,
            )
            response.raise_for_status()
        except requests.exceptions.RequestException as e:
            logger.critical(f"Response error raised: {e}")
            raise together.ResponseError(e)

        try:
            response_dict = response.json()
        except Exception as e:
            logger.critical(
                f"JSON Error raised: {e}\nResponse status code = {response.status_code}"
            )
            raise together.JSONError(e, http_status=response.status_code)

        return dict(response_dict)

    @classmethod
    def start(self, model: str) -> Dict[str, str]:
        model_url = urllib.parse.urljoin(
            together.api_base_instances, f"start?model={model}"
        )
        headers = {
            "Authorization": f"Bearer {together.api_key}",
            "accept": "application/json",
        }
        try:
            response = requests.post(
                model_url,
                headers=headers,
            )
            response.raise_for_status()
        except requests.exceptions.RequestException as e:
            logger.critical(f"Response error raised: {e}")
            raise together.ResponseError(e)

        try:
            response_dict = response.json()
        except Exception as e:
            logger.critical(
                f"JSON Error raised: {e}\nResponse status code = {response.status_code}"
            )
            raise together.JSONError(e, http_status=response.status_code)

        return dict(response_dict)

    @classmethod
    def stop(self, model: str) -> Dict[str, str]:
        model_url = urllib.parse.urljoin(
            together.api_base_instances, f"stop?model={model}"
        )
        headers = {
            "Authorization": f"Bearer {together.api_key}",
            "accept": "application/json",
        }
        try:
            response = requests.post(
                model_url,
                headers=headers,
            )
            response.raise_for_status()
        except requests.exceptions.RequestException as e:
            logger.critical(f"Response error raised: {e}")
            raise together.ResponseError(e)

        try:
            response_dict = response.json()
        except Exception as e:
            logger.critical(
                f"JSON Error raised: {e}\nResponse status code = {response.status_code}"
            )
            raise together.JSONError(e, http_status=response.status_code)

        return dict(response_dict)



import logging
from typing import Any, Dict, Optional

import together


class TogetherLogFormatter(logging.Formatter):
    grey = "\x1b[38;20m"
    yellow = "\x1b[33;20m"
    red = "\x1b[31;20m"
    bold_red = "\x1b[31;1m"
    reset = "\x1b[0m"
    log_format = (
        "%(asctime)s - %(name)s - %(levelname)s - %(message)s (%(filename)s:%(lineno)d)"
    )

    FORMATS = {
        logging.DEBUG: grey + log_format + reset,
        logging.INFO: grey + log_format + reset,
        logging.WARNING: yellow + log_format + reset,
        logging.ERROR: red + log_format + reset,
        logging.CRITICAL: bold_red + log_format + reset,
    }

    def format(self, record: logging.LogRecord) -> str:
        log_fmt = self.FORMATS.get(record.levelno)
        formatter = logging.Formatter(log_fmt, datefmt="%m-%d-%Y %H:%M:%S")
        return formatter.format(record)


# Setup logging
def get_logger(
    name: str,
    logger: Optional[logging.Logger] = None,
    log_level: str = together.log_level,
) -> logging.Logger:
    if logger is None:
        logger = logging.getLogger(name)

        logger.setLevel(log_level)

        lg_format = logging.StreamHandler()
        lg_format.setLevel(logging.DEBUG)
        lg_format.setFormatter(TogetherLogFormatter())

        logger.addHandler(lg_format)

    return logger


def verify_api_key(logger: Optional[logging.Logger] = None) -> None:
    if logger is None:
        logger = get_logger(str(__name__), log_level=together.log_level)
    if together.api_key is None:
        raise together.AuthenticationError(
            "TOGETHER_API_KEY not found. Please set it as an environment variable or set it with together.api_key"
        )


def extract_time(json_obj: Dict[str, Any]) -> int:
    try:
        return int(json_obj["created_at"])
    except KeyError:
        return 0



from __future__ import annotations

import argparse
import json
import logging
import re
import sys
from typing import Any, Dict, List

import together
from together import Complete, get_logger


def add_parser(
    subparsers: argparse._SubParsersAction[argparse.ArgumentParser],
) -> None:
    COMMAND_NAME = "complete"
    subparser = subparsers.add_parser(COMMAND_NAME)

    subparser.add_argument(
        "prompt",
        metavar="PROMPT",
        default=None,
        type=str,
        help="A string providing context for the model to complete.",
    )

    subparser.add_argument(
        "--model",
        "-m",
        default=together.default_text_model,
        type=str,
        help=f"The name of the model to query. Default={together.default_text_model}",
    )

    subparser.add_argument(
        "--no-stream",
        default=False,
        action="store_true",
        help="Indicates wether to disable streaming",
    )

    subparser.add_argument(
        "--max-tokens",
        default=128,
        type=int,
        help="Maximum number of tokens to generate. Default=128",
    )
    subparser.add_argument(
        "--stop",
        default=["<human>"],
        nargs="+",
        type=str,
        metavar="STOP_WORD",
        help="Strings that will truncate (stop) text generation. Default='<human>'",
    )
    subparser.add_argument(
        "--temperature",
        default=0.7,
        type=float,
        help="Determines the degree of randomness in the response. Default=0.7",
    )
    subparser.add_argument(
        "--top-p",
        default=0.7,
        type=float,
        help="Used to dynamically adjust the number of choices for each predicted token based on the cumulative probabilities. Default=0.7",
    )
    subparser.add_argument(
        "--top-k",
        default=50,
        type=int,
        help="Used to limit the number of choices for the next predicted word or token. Default=50",
    )
    subparser.add_argument(
        "--repetition-penalty",
        default=None,
        type=float,
        help="Controls the diversity of generated text by reducing the likelihood of repeated sequences. Higher values decrease repetition.",
    )
    subparser.add_argument(
        "--logprobs",
        default=None,
        type=int,
        help="Specifies how many top token log probabilities are included in the response for each token generation step.",
    )
    subparser.add_argument(
        "--raw",
        default=False,
        action="store_true",
        help="temperature for the LM",
    )
    subparser.set_defaults(func=_run_complete)


def _enforce_stop_tokens(text: str, stop: List[str]) -> str:
    """Cut off the text as soon as any stop words occur."""
    return re.split("|".join(stop), text)[0]


def no_streamer(
    args: argparse.Namespace, response: Dict[str, Any], logger: logging.Logger
) -> None:
    if args.raw:
        print(json.dumps(response, indent=4))
        sys.exit()

    if "output" in response.keys():
        try:
            text = str(response["output"]["choices"][0]["text"])
        except Exception:
            try:
                logger.critical(f"Error raised: {response['output']['error']}")
                raise together.ResponseError(response["output"]["error"])
            except Exception as e:
                logger.critical(f"Error raised: {e}")
                raise together.ResponseError(e)

        # if args.stop is not None:
        #    text = _enforce_stop_tokens(text, args.stop)

    elif "error" in response.keys():
        if response["error"] == "Returned error: no instance":
            logger.critical(
                f"No running instances for {args.model}. You can start an instance by navigating to the Together Playground at api.together.xyz"
            )
            raise together.InstanceError(model=args.model)
        else:
            logger.critical(f"Error raised: {response['error']}")

    else:
        logger.critical("Unknown response received")
        raise together.ResponseError("Unknown response received. Please try again.")

    print(text.strip())


def _run_complete(args: argparse.Namespace) -> None:
    logger = get_logger(__name__, log_level=args.log)

    complete = Complete()

    if args.no_stream:
        response = complete.create(
            prompt=args.prompt,
            model=args.model,
            max_tokens=args.max_tokens,
            stop=args.stop,
            temperature=args.temperature,
            top_p=args.top_p,
            top_k=args.top_k,
            repetition_penalty=args.repetition_penalty,
            logprobs=args.logprobs,
        )
        no_streamer(args, response, logger)
    else:
        for text in complete.create_streaming(
            prompt=args.prompt,
            model=args.model,
            max_tokens=args.max_tokens,
            stop=args.stop,
            temperature=args.temperature,
            top_p=args.top_p,
            top_k=args.top_k,
            repetition_penalty=args.repetition_penalty,
        ):
            print(text, end="", flush=True)
        print("\n")



from typing import Any, Dict, Optional

import requests

import together
from together import get_logger, verify_api_key


logger = get_logger(str(__name__), log_level=together.log_level)


class Image:
    def __init__(
        self,
    ) -> None:
        verify_api_key(logger)

    @classmethod
    def create(
        self,
        prompt: str,
        model: Optional[str] = "",
        steps: Optional[int] = 20,
        seed: Optional[int] = 42,
        results: Optional[int] = 1,
        height: Optional[int] = 256,
        width: Optional[int] = 256,
        negative_prompt: Optional[str] = "",
    ) -> Dict[str, Any]:
        if model == "":
            model = together.default_image_model

        parameter_payload = {
            "model": model,
            "prompt": prompt,
            "n": results,
            "mode": "text2img",
            "steps": steps,
            "seed": seed,
            "height": height,
            "width": width,
            "negative_prompt": negative_prompt,
        }

        # HTTP headers for authorization
        headers = {
            "Authorization": f"Bearer {together.api_key}",
            "Content-Type": "application/json",
            "User-Agent": together.user_agent,
        }

        # send request
        try:
            response = requests.post(
                together.api_base_complete,
                headers=headers,
                json=parameter_payload,
            )
        except requests.exceptions.RequestException as e:
            logger.critical(f"Response error raised: {e}")
            raise together.ResponseError(e)

        if response.status_code == 429:
            logger.critical(
                f"No running instances for {model}. You can start an instance by navigating to the Together Playground at api.together.ai"
            )
            raise together.InstanceError(model=model)

        response.raise_for_status()

        try:
            response_json = dict(response.json())
        except Exception as e:
            logger.critical(
                f"JSON Error raised: {e}\nResponse status code = {response.status_code}"
            )
            raise together.JSONError(e, http_status=response.status_code)
        return response_json



import os
import urllib.parse

from .config import (
    finetune_model_names,
    jokes_list,
    min_samples,
    model_info_dict,
)
from .version import VERSION


version = VERSION

user_agent = f"TogetherPythonOfficial/{version}"

api_key = os.environ.get("TOGETHER_API_KEY", None)

api_base = "https://api.together.xyz/"
api_base_complete = urllib.parse.urljoin(api_base, "/api/inference")
api_base_files = urllib.parse.urljoin(api_base, "/v1/files/")
api_base_finetune = urllib.parse.urljoin(api_base, "/v1/fine-tunes/")
api_base_instances = urllib.parse.urljoin(api_base, "instances/")

default_text_model = "togethercomputer/RedPajama-INCITE-7B-Chat"
default_image_model = "runwayml/stable-diffusion-v1-5"
log_level = "WARNING"

from .utils.utils import get_logger, verify_api_key, extract_time  # noqa
from .models import Models
from .complete import Complete
from .error import *
from .files import Files
from .finetune import Finetune
from .image import Image


__all__ = [
    "api_key",
    "api_base",
    "api_base_complete",
    "api_base_files",
    "api_base_finetune",
    "api_base_instances",
    "default_text_model",
    "default_image_model",
    "get_logger",
    "verify_api_key",
    "extract_time",
    "Models",
    "Complete",
    "Files",
    "Finetune",
    "Image",
    "model_info_dict",
    "finetune_model_names",
    "min_samples",
    "jokes_list",
]



import json
from typing import Any, Dict, Iterator, List, Optional

import requests
import sseclient  # type: ignore

import together
from together import get_logger, verify_api_key


logger = get_logger(str(__name__), log_level=together.log_level)


class Complete:
    def __init__(
        self,
    ) -> None:
        verify_api_key(logger)

    @classmethod
    def create(
        self,
        prompt: str,
        model: Optional[str] = "",
        max_tokens: Optional[int] = 128,
        stop: Optional[List[str]] = [],
        temperature: Optional[float] = 0.7,
        top_p: Optional[float] = 0.7,
        top_k: Optional[int] = 50,
        repetition_penalty: Optional[float] = None,
        logprobs: Optional[int] = None,
    ) -> Dict[str, Any]:
        if model == "":
            model = together.default_text_model

        parameter_payload = {
            "model": model,
            "prompt": prompt,
            "top_p": top_p,
            "top_k": top_k,
            "temperature": temperature,
            "max_tokens": max_tokens,
            "stop": stop,
            "repetition_penalty": repetition_penalty,
            "logprobs": logprobs,
        }

        # HTTP headers for authorization
        headers = {
            "Authorization": f"Bearer {together.api_key}",
            "Content-Type": "application/json",
            "User-Agent": together.user_agent,
        }

        # send request
        try:
            response = requests.post(
                together.api_base_complete,
                headers=headers,
                json=parameter_payload,
            )
        except requests.exceptions.RequestException as e:
            logger.critical(f"Response error raised: {e}")
            raise together.ResponseError(e)

        if response.status_code == 429:
            logger.critical(
                f"""No running instances for {model}.
                You can start an instance with one of the following methods:
                  1. navigating to the Together Playground at api.together.ai
                  2. starting one in python using together.Models.start(model_name)
                  3. `$ together models start <MODEL_NAME>` at the command line.
                See `together.Models.list()` in python or `$ together models list` in command line
                to get an updated list of valid model names.
                """
            )
            raise together.InstanceError(model=model)

        response.raise_for_status()

        try:
            response_json = dict(response.json())

        except Exception as e:
            logger.critical(
                f"Error raised: {e}\nResponse status code = {response.status_code}"
            )
            raise together.JSONError(e, http_status=response.status_code)
        return response_json

    @classmethod
    def create_streaming(
        self,
        prompt: str,
        model: Optional[str] = "",
        max_tokens: Optional[int] = 128,
        stop: Optional[str] = None,
        temperature: Optional[float] = 0.7,
        top_p: Optional[float] = 0.7,
        top_k: Optional[int] = 50,
        repetition_penalty: Optional[float] = None,
    ) -> Iterator[str]:
        """
        Prints streaming responses and returns the completed text.
        """

        if model == "":
            model = together.default_text_model

        parameter_payload = {
            "model": model,
            "prompt": prompt,
            "top_p": top_p,
            "top_k": top_k,
            "temperature": temperature,
            "max_tokens": max_tokens,
            "stop": stop,
            "repetition_penalty": repetition_penalty,
            "stream_tokens": True,
        }
        # HTTP headers for authorization
        headers = {
            "Authorization": f"Bearer {together.api_key}",
            "Content-Type": "application/json",
            "User-Agent": together.user_agent,
        }

        # send request
        try:
            response = requests.post(
                together.api_base_complete,
                headers=headers,
                json=parameter_payload,
                stream=True,
            )
        except requests.exceptions.RequestException as e:
            logger.critical(f"Response error raised: {e}")
            raise together.ResponseError(e)

        if response.status_code == 200:
            output = ""
            client = sseclient.SSEClient(response)
            for event in client.events():
                if event.data != "[DONE]":
                    text = json.loads(event.data)["choices"][0]["text"]
                    output += text
                    yield text
        elif response.status_code == 429:
            logger.critical(
                f"""No running instances for {model}.
                You can start an instance with one of the following methods:
                  1. navigating to the Together Playground at api.together.ai
                  2. starting one in python using together.Models.start(model_name)
                  3. `$ together models start <MODEL_NAME>` at the command line.
                See `together.Models.list()` in python or `$ together models list` in command line
                to get an updated list of valid model names.
                """
            )
            raise together.InstanceError(model=model)
        else:
            logger.critical(
                f"Unknown error raised.\nResponse status code = {response.status_code}"
            )
            response.raise_for_status()
            raise together.ResponseError(http_status=response.status_code)



import json
import os
import posixpath
import urllib.parse
from typing import Any, Dict, List, Mapping, Optional, Union, cast

import requests
from tqdm import tqdm
from tqdm.utils import CallbackIOWrapper

import together
from together import get_logger, verify_api_key


logger = get_logger(str(__name__), log_level=together.log_level)

# the number of bytes in a gigabyte, used to convert bytes to GB for readable comparison
NUM_BYTES_IN_GB = 2**30

# maximum number of GB sized files we support finetuning for
MAX_FT_GB = 4.9


class Files:
    def __init__(
        self,
    ) -> None:
        verify_api_key(logger)

    @classmethod
    def list(self) -> Dict[str, List[Dict[str, Union[str, int]]]]:
        headers = {
            "Authorization": f"Bearer {together.api_key}",
            "User-Agent": together.user_agent,
        }

        # send request
        try:
            response = requests.get(together.api_base_files, headers=headers)
            response.raise_for_status()
        except requests.exceptions.RequestException as e:
            logger.critical(f"Response error raised: {e}")
            raise together.ResponseError(e)
        try:
            response_json = dict(response.json())
        except Exception as e:
            logger.critical(
                f"JSON Error raised: {e}\nResponse status code = {response.status_code}"
            )
            raise together.JSONError(e, http_status=response.status_code)

        return response_json

    @classmethod
    def check(self, file: str, model: Optional[str] = None) -> Dict[str, object]:
        return check_json(file, model)

    @classmethod
    def upload(
        self,
        file: str,
        check: bool = True,
        model: Optional[str] = None,
    ) -> Mapping[str, Union[str, int, Any]]:
        data = {"purpose": "fine-tune", "file_name": os.path.basename(file)}

        headers = {
            "Authorization": f"Bearer {together.api_key}",
            "User-Agent": together.user_agent,
        }

        if check:
            report_dict = check_json(file, model)
            if not report_dict["is_check_passed"]:
                print(report_dict)
                raise together.FileTypeError("Invalid file supplied. Failed to upload.")
        else:
            report_dict = {}

        session = requests.Session()

        init_endpoint = together.api_base_files[:-1]

        logger.debug(
            f"Upload file POST request: data={data}, headers={headers}, URL={init_endpoint}, allow_redirects=False"
        )
        try:
            response = session.post(
                init_endpoint,
                data=data,
                headers=headers,
                allow_redirects=False,
            )

            logger.debug(f"Response text: {response.text}")
            logger.debug(f"Response header: {response.headers}")
            logger.debug(f"Response status code: {response.status_code}")

            if response.status_code == 401:
                logger.critical(
                    "This job would exceed your free trial credits. Please upgrade to a paid account through Settings -> Billing on api.together.ai to continue."
                )
                raise together.AuthenticationError(
                    "This job would exceed your free trial credits. Please upgrade to a paid account through Settings -> Billing on api.together.ai to continue."
                )
            elif response.status_code != 302:
                logger.critical(
                    f"Unexpected error raised by endpoint. Response status code: {response.status_code}"
                )
                raise together.ResponseError(
                    "Unexpected error raised by endpoint.",
                    http_status=response.status_code,
                )

            r2_signed_url = response.headers["Location"]
            file_id = response.headers["X-Together-File-Id"]

            logger.info(f"R2 Signed URL: {r2_signed_url}")
            logger.info("File-ID")

            logger.info("Uploading file...")

            file_size = os.stat(file).st_size
            progress_bar = tqdm(
                total=file_size, unit="B", unit_scale=True, unit_divisor=1024
            )
            progress_bar.set_description(f"Uploading {file}")
            with open(file, "rb") as f:
                wrapped_file = CallbackIOWrapper(progress_bar.update, f, "read")
                response = requests.put(r2_signed_url, data=wrapped_file)
                response.raise_for_status()

            logger.info("File uploaded.")
            logger.debug(f"status code: {response.status_code}")
            logger.info("Processing file...")
            preprocess_url = urllib.parse.urljoin(
                together.api_base_files, f"{file_id}/preprocess"
            )

            response = session.post(
                preprocess_url,
                headers=headers,
            )

            logger.info("File processed")
            logger.debug(f"Status code: {response.status_code}")

        except Exception as e:
            logger.critical(f"Response error raised: {e}")
            raise together.ResponseError(e)

        # output_dict["filename"] = os.path.basename(file)
        # output_dict["id"] = str(file_id)
        # output_dict["object"] = "file"

        return {
            "filename": os.path.basename(file),
            "id": str(file_id),
            "object": "file",
            "report_dict": report_dict,
        }

    @classmethod
    def delete(self, file_id: str) -> Dict[str, str]:
        delete_url = urllib.parse.urljoin(together.api_base_files, file_id)

        headers = {
            "Authorization": f"Bearer {together.api_key}",
            "User-Agent": together.user_agent,
        }

        # send request
        try:
            response = requests.delete(delete_url, headers=headers)
            response.raise_for_status()
        except requests.exceptions.RequestException as e:
            logger.critical(f"Response error raised: {e}")
            raise together.ResponseError(e)

        try:
            response_json = dict(response.json())
        except Exception as e:
            logger.critical(
                f"JSON Error raised: {e}\nResponse status code = {response.status_code}"
            )
            raise together.JSONError(e, http_status=response.status_code)

        return response_json

    @classmethod
    def retrieve(self, file_id: str) -> Dict[str, Union[str, int]]:
        retrieve_url = urllib.parse.urljoin(together.api_base_files, file_id)

        logger.info(f"Retrieve URL: {retrieve_url}")

        headers = {
            "Authorization": f"Bearer {together.api_key}",
            "User-Agent": together.user_agent,
        }

        # send request
        try:
            response = requests.get(retrieve_url, headers=headers)
            response.raise_for_status()
        except requests.exceptions.RequestException as e:
            logger.critical(f"Response error raised: {e}")
            raise together.ResponseError(e)

        try:
            response_json = dict(response.json())
        except Exception as e:
            logger.critical(
                f"JSON Error raised: {e}\nResponse status code = {response.status_code}"
            )
            raise together.JSONError(e, http_status=response.status_code)

        return response_json

    @classmethod
    def retrieve_content(self, file_id: str, output: Union[str, None] = None) -> str:
        if output is None:
            output = file_id + ".jsonl"

        relative_path = posixpath.join(file_id, "content")
        retrieve_url = urllib.parse.urljoin(together.api_base_files, relative_path)

        headers = {
            "Authorization": f"Bearer {together.api_key}",
            "User-Agent": together.user_agent,
        }

        # send request
        try:
            session = requests.Session()

            response = session.get(retrieve_url, headers=headers, stream=True)
            response.raise_for_status()

            total_size_in_bytes = int(response.headers.get("content-length", 0))
            block_size = 1024 * 1024  # 1 MB
            progress_bar = tqdm(total=total_size_in_bytes, unit="iB", unit_scale=True)
            progress_bar.set_description(f"Downloading {output}")

            with open(output, "wb") as file:
                for chunk in response.iter_content(block_size):
                    progress_bar.update(len(chunk))
                    file.write(chunk)

            progress_bar.close()

            if total_size_in_bytes != 0 and progress_bar.n != total_size_in_bytes:
                logger.warning(
                    "Caution: Downloaded file size does not match remote file size."
                )

        except requests.exceptions.RequestException as e:  # This is the correct syntax
            logger.critical(f"Response error raised: {e}")
            raise together.ResponseError(e)

        return output  # this should be null

    @classmethod
    def save_jsonl(
        self, data: Dict[str, str], output_path: str, append: bool = False
    ) -> None:
        """
        Write list of objects to a JSON lines file.
        """
        mode = "a+" if append else "w"
        with open(output_path, mode, encoding="utf-8") as f:
            for line in data:
                json_record = json.dumps(line, ensure_ascii=False)
                f.write(json_record + "\n")
        print("Wrote {} records to {}".format(len(data), output_path))

    @classmethod
    def load_jsonl(self, input_path: str) -> List[Dict[str, str]]:
        """
        Read list of objects from a JSON lines file.
        """
        data = []
        with open(input_path, "r", encoding="utf-8") as f:
            for line in f:
                data.append(json.loads(line.rstrip("\n|\r")))
        print("Loaded {} records from {}".format(len(data), input_path))
        return data


def check_json(
    file: str,
    model: Optional[str] = None,
) -> Dict[str, object]:
    report_dict = {
        "is_check_passed": True,
        "model_special_tokens": "we are not yet checking end of sentence tokens for this model",
    }
    num_samples_w_eos_token = 0

    model_info_dict = cast(Dict[str, Any], together.model_info_dict)

    eos_token = None
    if model is not None and model in model_info_dict:
        if "eos_token" in model_info_dict[model]:
            eos_token = model_info_dict[model]["eos_token"]
            report_dict[
                "model_special_tokens"
            ] = f"the end of sentence token for this model is {eos_token}"

    if not os.path.isfile(file):
        report_dict["file_present"] = f"File not found at given file path {file}"
        report_dict["is_check_passed"] = False
    else:
        report_dict["file_present"] = "File found"

    file_size = os.stat(file).st_size

    if file_size > MAX_FT_GB * NUM_BYTES_IN_GB:
        report_dict[
            "file_size"
        ] = f"File size {round(file_size / NUM_BYTES_IN_GB ,3)} GB is greater than our limit of 4.9 GB"
        report_dict["is_check_passed"] = False
    else:
        report_dict["file_size"] = f"File size {round(file_size / (2**30) ,3)} GB"

    with open(file) as f:
        try:
            for idx, line in enumerate(f):
                json_line = json.loads(line)  # each line in jsonlines should be a json

                if not isinstance(json_line, dict):
                    report_dict["line_type"] = (
                        "Valid json not found in one or more lines in JSONL file."
                        'Example of valid json: {"text":"my sample string"}.'
                        "see https://docs.together.ai/docs/fine-tuning."
                        f"The first line where this occur is line {idx+1}, where 1 is the first line."
                        f"{str(line)}"
                    )
                    report_dict["is_check_passed"] = False

                if "text" not in json_line:
                    report_dict["text_field"] = (
                        f'No "text" field was found on line {idx+1} of the the input file.'
                        'Expected format: {"text":"my sample string"}.'
                        "see https://docs.together.ai/docs/fine-tuning for more information."
                        f"{str(line)}"
                    )
                    report_dict["is_check_passed"] = False
                else:
                    # check to make sure the value of the "text" key is a string
                    if not isinstance(json_line["text"], str):
                        report_dict["key_value"] = (
                            f'Unexpected, value type for "text" key on line {idx+1} of the input file.'
                            'The value type of the "text" key must be a string.'
                            'Expected format: {"text":"my sample string"}'
                            "See https://docs.together.ai/docs/fine-tuning for more information."
                            f"{str(line)}"
                        )

                        report_dict["is_check_passed"] = False

                    elif eos_token:
                        if eos_token in json_line["text"]:
                            num_samples_w_eos_token += 1

            # make sure this is outside the for idx, line in enumerate(f): for loop
            if idx + 1 < together.min_samples:
                report_dict["min_samples"] = (
                    f"Processing {file} resulted in only {idx+1} samples. "
                    f"Our minimum is {together.min_samples} samples. "
                )
                report_dict["is_check_passed"] = False
            else:
                report_dict["num_samples"] = idx + 1

        except ValueError:
            report_dict["load_json"] = (
                f"File should be a valid jsonlines (.jsonl) with a json in each line."
                'Example of valid json: {"text":"my sample string"}'
                "Valid json not found in one or more lines in file."
                "see https://docs.together.ai/docs/fine-tuning."
                f"The first line where this occur is line {idx+1}, where 1 is the first line."
                f"{str(line)}"
            )
            report_dict["is_check_passed"] = False

    report_dict["num_samples_w_eos_token"] = num_samples_w_eos_token

    return report_dict



import posixpath
import urllib.parse
from typing import Any, Dict, List, Optional, Union

import requests
from tqdm import tqdm

import together
from together import Files, get_logger, verify_api_key


logger = get_logger(str(__name__), log_level=together.log_level)


# this will change soon to be data driven and give a clearer estimate
def model_param_count(name: str) -> int:
    pcount = {
        "togethercomputer/RedPajama-INCITE-7B-Chat": 6857302016,
        "togethercomputer/RedPajama-INCITE-7B-Base": 6857302016,
        "togethercomputer/RedPajama-INCITE-7B-Instruct": 6857302016,
        "togethercomputer/RedPajama-INCITE-Chat-3B-v1": 2775864320,
        "togethercomputer/RedPajama-INCITE-Base-3B-v1": 2775864320,
        "togethercomputer/RedPajama-INCITE-Instruct-3B-v1": 2775864320,
        "togethercomputer/Pythia-Chat-Base-7B": 6857302016,
        "togethercomputer/llama-2-7b": 6738415616,
        "togethercomputer/llama-2-7b-chat": 6738415616,
        "togethercomputer/llama-2-13b": 13015864320,
        "togethercomputer/llama-2-13b-chat": 13015864320,
        "togethercomputer/LLaMA-2-7B-32K": 6738415616,
        "togethercomputer/Llama-2-7B-32K-Instruct": 6738415616,
        "togethercomputer/CodeLlama-7b": 6738546688,
        "togethercomputer/CodeLlama-7b-Python": 6738546688,
        "togethercomputer/CodeLlama-7b-Instruct": 6738546688,
        "togethercomputer/CodeLlama-13b": 13016028160,
        "togethercomputer/CodeLlama-13b-Python": 13016028160,
        "togethercomputer/CodeLlama-13b-Instruct": 13016028160,
        "togethercomputer/llama-2-70b": 68976648192,
        "togethercomputer/llama-2-70b-chat": 68976648192,
    }
    try:
        return pcount[name]
    except Exception:
        return 0


class Finetune:
    def __init__(
        self,
    ) -> None:
        verify_api_key(logger)

    @classmethod
    def create(
        self,
        training_file: str,  # training file_id
        # validation_file: Optional[str] = None,  # validation file_id
        model: str,
        n_epochs: int = 1,
        n_checkpoints: Optional[int] = 1,
        batch_size: Optional[int] = 32,
        learning_rate: Optional[float] = 0.00001,
        # warmup_steps: Optional[int] = 0,
        # train_warmup_steps: Optional[int] = 0,
        # seq_length: Optional[int] = 2048,
        # seed: Optional[int] = 42,
        # fp16: Optional[bool] = True,
        # checkpoint_steps: Optional[int] = None,
        suffix: Optional[
            str
        ] = None,  # resulting finetuned model name will include the suffix
        estimate_price: bool = False,
        wandb_api_key: Optional[str] = None,
    ) -> Dict[Any, Any]:
        if n_epochs is None or n_epochs < 1:
            logger.fatal("The number of epochs must be specified")
            raise ValueError("n_epochs is required")

        # Validate parameters
        if n_checkpoints is None:
            n_checkpoints = 1
        elif n_checkpoints < 1:
            n_checkpoints = 1
            logger.warning(
                f"The number of checkpoints must be >= 1, setting to {n_checkpoints}"
            )
        elif n_checkpoints > n_epochs:
            n_checkpoints = n_epochs
            logger.warning(
                f"The number of checkpoints must be < the number of epochs, setting to {n_checkpoints}"
            )

        if (
            model
            in ["togethercomputer/llama-2-70b", "togethercomputer/llama-2-70b-chat"]
            and batch_size != 144
        ):
            raise ValueError(
                f"Batch size must be 144 for {model} model. Please set batch size to 144"
            )

        if batch_size is None:
            batch_size = 32
        elif batch_size < 4:
            raise ValueError("Batch size must be >= 4.")

        # TODO: REMOVE THIS CHECK WHEN WE HAVE CHECKPOINTING WORKING FOR 70B models
        if n_checkpoints > 1 and model in [
            "togethercomputer/llama-2-70b",
            "togethercomputer/llama-2-70b-chat",
        ]:
            raise ValueError(
                "Saving checkpoints during training currently not supported for {model}.  Please set the number of checkpoints to 1"
            )

        parameter_payload = {
            "training_file": training_file,
            # "validation_file": validation_file,
            "model": model,
            "n_epochs": n_epochs,
            "n_checkpoints": n_checkpoints,
            "batch_size": batch_size,
            "learning_rate": learning_rate,
            # "warmup_steps": warmup_steps,
            # "train_warmup_steps": train_warmup_steps,
            # "seq_length": seq_length,
            # "seed": seed,
            # "fp16": fp16,
            "suffix": suffix,
            "wandb_key": wandb_api_key,
        }

        # check if model name is one of the models available for finetuning
        if parameter_payload["model"] not in together.finetune_model_names:
            logger.warning(
                "The finetune model name must be one of the subset of models available for finetuning. "
                "Here is a list of those models https://docs.together.ai/docs/models-fine-tuning"
            )

        # check if training_file is the string id of a previously uploaded file
        uploaded_files = Files.list()
        file_ids = [f["id"] for f in uploaded_files["data"]]
        if parameter_payload["training_file"] not in file_ids:
            training_file_feedback = (
                "training_file refers to a file identifier of an uploaded training file, not a local file path. "
                "A list of uploaded files and file identifiers can be retrieved with `together.Files.list()` Python API or "
                "$ together files list` CLI. A training file can be uploaded using `together.Files.upload(file ='/path/to/file')"
                "Python API or `$ together files upload <FILE_PATH>` CLI."
            )
            logger.critical(training_file_feedback)
            raise together.FileTypeError(training_file_feedback)

        if estimate_price:
            param_size = model_param_count(model)
            if param_size == 0:
                error = f"Unknown model {model}.  Cannot estimate price.  Please check the name of the model"
                raise together.FileTypeError(error)

            for file in uploaded_files["data"]:
                if file["id"] == parameter_payload["training_file"]:
                    ## This is the file
                    byte_count = file["bytes"]
                    token_estimate = int(int(file["bytes"]) / 4)
                    data = {
                        "method": "together_getPrice",
                        "params": [
                            model,
                            "FT",
                            {
                                "tokens": token_estimate,
                                "epochs": n_epochs,
                                "parameters": model_param_count(model),
                            },
                        ],
                        "id": 1,
                    }
                    r = requests.post("https://computer.together.xyz/", json=data)
                    estimate = r.json()["result"]["total"]
                    estimate /= 1000000000
                    training_file_feedback = f"A rough price estimate for this job is ${estimate:.2f} USD. The estimated number of tokens is {token_estimate} tokens. Accurate pricing is not available until full tokenization has been performed. The actual price might be higher or lower depending on how the data is tokenized. Our token estimate is based on the number of bytes in the training file, {byte_count} bytes, divided by an average token length of 4 bytes. We currently have a per job minimum of $5.00 USD."
                    print(training_file_feedback)
                    exit()

        # Send POST request to SUBMIT FINETUNE JOB
        # HTTP headers for authorization
        headers = {
            "Authorization": f"Bearer {together.api_key}",
            "Content-Type": "application/json",
            "User-Agent": together.user_agent,
        }
        try:
            response = requests.post(
                together.api_base_finetune, headers=headers, json=parameter_payload
            )
            response.raise_for_status()
        except requests.exceptions.RequestException as e:
            logger.critical(f"Response error raised: {e}")
            raise together.ResponseError(e)

        try:
            response_json = dict(response.json())
        except Exception as e:
            logger.critical(
                f"JSON Error raised: {e}\nResponse status code = {response.status_code}"
            )
            raise together.JSONError(e, http_status=response.status_code)

        return response_json

    @classmethod
    def list(self) -> Dict[Any, Any]:
        verify_api_key(logger)
        headers = {
            "Authorization": f"Bearer {together.api_key}",
            "User-Agent": together.user_agent,
        }

        # send request
        try:
            response = requests.get(together.api_base_finetune, headers=headers)
            response.raise_for_status()
        except requests.exceptions.RequestException as e:
            logger.critical(f"Response error raised: {e}")
            raise together.ResponseError(e)

        try:
            response_json = dict(response.json())
        except Exception as e:
            logger.critical(
                f"JSON Error raised: {e}\nResponse status code = {response.status_code}"
            )
            raise together.JSONError(e, http_status=response.status_code)

        return response_json

    @classmethod
    def retrieve(self, fine_tune_id: str) -> Dict[Any, Any]:
        retrieve_url = urllib.parse.urljoin(together.api_base_finetune, fine_tune_id)

        headers = {
            "Authorization": f"Bearer {together.api_key}",
            "User-Agent": together.user_agent,
        }

        # send request
        try:
            response = requests.get(retrieve_url, headers=headers)
            response.raise_for_status()
        except requests.exceptions.RequestException as e:
            logger.critical(f"Response error raised: {e}")
            raise together.ResponseError(e)

        try:
            response_json = dict(response.json())
        except Exception as e:
            logger.critical(
                f"JSON Error raised: {e}\nResponse status code = {response.status_code}"
            )
            raise together.JSONError(e, http_status=response.status_code)

        return response_json

    @classmethod
    def cancel(self, fine_tune_id: str) -> Dict[Any, Any]:
        relative_path = posixpath.join(fine_tune_id, "cancel")
        retrieve_url = urllib.parse.urljoin(together.api_base_finetune, relative_path)

        headers = {
            "Authorization": f"Bearer {together.api_key}",
            "User-Agent": together.user_agent,
        }

        # send request
        try:
            response = requests.post(retrieve_url, headers=headers)
            response.raise_for_status()
        except requests.exceptions.RequestException as e:
            logger.critical(f"Response error raised: {e}")
            raise together.ResponseError(e)

        try:
            response_json = dict(response.json())
        except Exception as e:
            logger.critical(
                f"JSON Error raised: {e}\nResponse status code = {response.status_code}"
            )
            raise together.JSONError(e, http_status=response.status_code)

        return response_json

    @classmethod
    def list_events(self, fine_tune_id: str) -> Dict[Any, Any]:
        # TODO enable stream
        relative_path = posixpath.join(fine_tune_id, "events")
        retrieve_url = urllib.parse.urljoin(together.api_base_finetune, relative_path)

        headers = {
            "Authorization": f"Bearer {together.api_key}",
            "User-Agent": together.user_agent,
        }

        # send request
        try:
            response = requests.get(retrieve_url, headers=headers)
            response.raise_for_status()
        except requests.exceptions.RequestException as e:
            logger.critical(f"Response error raised: {e}")
            raise together.ResponseError(e)

        try:
            response_json = dict(response.json())
        except Exception as e:
            logger.critical(
                f"JSON Error raised: {e}\nResponse status code = {response.status_code}"
            )
            raise together.JSONError(e, http_status=response.status_code)

        return response_json

    @classmethod
    def get_checkpoints(self, fine_tune_id: str) -> List[Dict[str, Any]]:
        try:
            finetune_events = list(self.retrieve(fine_tune_id=fine_tune_id)["events"])
        except Exception as e:
            logger.critical(f"Response error raised: {e}")
            raise together.ResponseError(e)

        saved_events = [i for i in finetune_events if i["type"] in ["CHECKPOINT_SAVE"]]

        return saved_events

    @classmethod
    def get_job_status(self, fine_tune_id: str) -> str:
        try:
            job_status = str(self.retrieve(fine_tune_id=fine_tune_id)["status"])
        except Exception as e:
            logger.critical(f"Response error raised: {e}")
            raise together.ResponseError(e)

        return job_status

    @classmethod
    def is_final_model_available(self, fine_tune_id: str) -> bool:
        try:
            finetune_events = list(self.retrieve(fine_tune_id=fine_tune_id)["events"])
        except Exception as e:
            logger.critical(f"Response error raised: {e}")
            raise together.ResponseError(e)

        for i in finetune_events:
            if i["type"] in ["JOB_COMPLETE", "JOB_ERROR"]:
                if i["checkpoint_path"] != "":
                    return False
                else:
                    return True
        return False

    @classmethod
    def download(
        self,
        fine_tune_id: str,
        output: Union[str, None] = None,
        step: int = -1,
    ) -> str:
        # default to model_output_path name
        model_file_path = urllib.parse.urljoin(
            together.api_base_finetune,
            f"/api/finetune/downloadfinetunefile?ft_id={fine_tune_id}",
        )
        if step != -1:
            model_file_path += f"&checkpoint_step={step}"

        logger.info(f"Downloading weights from {model_file_path}...")

        headers = {
            "Authorization": f"Bearer {together.api_key}",
            "User-Agent": together.user_agent,
        }

        try:
            session = requests.Session()

            response = session.get(model_file_path, headers=headers, stream=True)
            response.raise_for_status()

            if output is None:
                content_type = str(response.headers.get("content-type"))

                output = self.retrieve(fine_tune_id)["model_output_path"].split("/")[-1]

                if step != -1:
                    output += f"-checkpoint-{step}"

                if "x-tar" in content_type.lower():
                    output += ".tar.gz"
                elif "zstd" in content_type.lower() or step != -1:
                    output += ".tar.zst"
                else:
                    raise together.ResponseError(
                        f"Unknown file type {content_type} found. Aborting download."
                    )

            total_size_in_bytes = int(response.headers.get("content-length", 0))
            block_size = 1024 * 1024  # 1 MB
            progress_bar = tqdm(total=total_size_in_bytes, unit="iB", unit_scale=True)
            progress_bar.set_description(f"Downloading {output}")
            with open(output, "wb") as file:
                for chunk in response.iter_content(block_size):
                    progress_bar.update(len(chunk))
                    file.write(chunk)
            progress_bar.close()
            if total_size_in_bytes != 0 and progress_bar.n != total_size_in_bytes:
                logger.warning(
                    "Caution: Downloaded file size does not match remote file size."
                )
        except requests.exceptions.RequestException as e:  # This is the correct syntax
            logger.critical(f"Response error raised: {e}")
            raise together.ResponseError(e)

        return output  # this should be null

    # def delete_finetune_model(self, model: str) -> Dict[Any, Any]:
    #     model_url = "https://api.together.xyz/api/models"
    #     delete_url = urllib.parse.urljoin(model_url, model)

    #     headers = {
    #         "Authorization": f"Bearer {together.api_key}",
    #     }

    #     # send request
    #     try:
    #         response = requests.delete(delete_url, headers=headers)
    #     except requests.exceptions.RequestException as e:
    #         raise ValueError(f"Error raised by finetune endpoint: {e}")

    #     try:
    #         response_json = dict(response.json())
    #     except Exception as e:
    #         raise ValueError(
    #             f"JSON Error raised. \nResponse status code: {str(response.status_code)}"
    #         )

    #     return response_json



from __future__ import annotations

import argparse
import cmd

import together
import together.utils.conversation as convo
from together import Complete, get_logger


def add_parser(
    subparsers: argparse._SubParsersAction[argparse.ArgumentParser],
) -> None:
    COMMAND_NAME = "chat"
    subparser = subparsers.add_parser(COMMAND_NAME)

    subparser.add_argument(
        "--model",
        "-m",
        default=together.default_text_model,
        type=str,
        help=f"The name of the model to query. Default={together.default_text_model}",
    )
    subparser.add_argument(
        "--prompt_id",
        "-pid",
        default="<human>",
        type=str,
        help="Indicates start of prompt. Defaults to '<human>'. Examples '### User' or '[INST]'",
    )
    subparser.add_argument(
        "--response_id",
        "-rid",
        default="<bot>",
        type=str,
        help="Indicates start of response. Defaults to '<bot>'. Examples '### Assistant' or '[/INST]'",
    )

    subparser.add_argument(
        "--max-tokens",
        default=128,
        type=int,
        help="Maximum number of tokens to generate. Default=128",
    )
    subparser.add_argument(
        "--stop",
        default=["<human>"],
        nargs="+",
        type=str,
        help="Strings that will truncate (stop) text generation. Default='<human>'",
    )
    subparser.add_argument(
        "--temperature",
        default=0.7,
        type=float,
        help="Determines the degree of randomness in the response. Default=0.7",
    )
    subparser.add_argument(
        "--top-p",
        default=0.7,
        type=float,
        help="Used to dynamically adjust the number of choices for each predicted token based on the cumulative probabilities. Default=0.7",
    )
    subparser.add_argument(
        "--top-k",
        default=50,
        type=int,
        help="Used to limit the number of choices for the next predicted word or token. Default=50",
    )
    subparser.add_argument(
        "--repetition-penalty",
        default=None,
        type=float,
        help="Controls the diversity of generated text by reducing the likelihood of repeated sequences. Higher values decrease repetition.",
    )

    subparser.set_defaults(func=_run_complete)


class OpenChatKitShell(cmd.Cmd):
    intro = "Type /quit to quit, /help, or /? to list commands.\n"
    prompt = ">>> "

    def __init__(self, infer: Complete, args: argparse.Namespace) -> None:
        super().__init__()
        self.infer = infer
        self.args = args
        self.prompt_id = args.prompt_id
        self.response_id = args.response_id
        print(f"Loading {self.args.model}")

    def preloop(self) -> None:
        self._convo = convo.Conversation(self.prompt_id, self.response_id)

    def precmd(self, line: str) -> str:
        if line.startswith("/"):
            return line[1:]
        else:
            return "say " + line

    def do_say(self, arg: str) -> None:
        self._convo.push_human_turn(arg)
        output = ""
        for token in self.infer.create_streaming(
            prompt=self._convo.get_raw_prompt(),
            model=self.args.model,
            max_tokens=self.args.max_tokens,
            stop=self.args.stop,
            temperature=self.args.temperature,
            top_p=self.args.top_p,
            top_k=self.args.top_k,
            repetition_penalty=self.args.repetition_penalty,
        ):
            print(token, end="", flush=True)
            output += token
        print("\n")
        self._convo.push_model_response(output)

    def do_raw_prompt(self, arg: str) -> None:
        print(self._convo.get_raw_prompt())

    def do_reset(self, arg: str) -> None:
        self._convo = convo.Conversation(self.prompt_id, self.response_id)

    def do_quit(self, arg: str) -> bool:
        return True


def _run_complete(args: argparse.Namespace) -> None:
    get_logger(__name__, log_level=args.log)
    if args.prompt_id not in args.stop:
        args.stop.append(args.prompt_id)

    infer = Complete()

    OpenChatKitShell(infer, args).cmdloop()



from typing import Any, Dict, Optional, Union

from requests import RequestException


class TogetherException(Exception):
    def __init__(
        self,
        message: Optional[Union[Exception, str, RequestException]] = "",
        http_body: Optional[str] = None,
        http_status: Optional[int] = None,
        json_body: Optional[Any] = None,
        headers: Optional[Union[str, Dict[Any, Any]]] = None,
    ) -> None:
        super(TogetherException, self).__init__(message)

        if http_body and hasattr(http_body, "decode"):
            try:
                http_body = http_body.decode("utf-8")
            except BaseException:
                http_body = (
                    "<Could not decode body as utf-8. "
                    "Please contact us via email at support@together.ai>"
                )

        self._message = message
        self.http_body = http_body
        self.http_status = http_status
        self.json_body = json_body
        self.headers = headers or {}

    def __repr__(self) -> str:
        return "%s(message=%r, http_status=%r)" % (
            self.__class__.__name__,
            self._message,
            self.http_status,
        )


class AuthenticationError(TogetherException):
    pass


class ResponseError(TogetherException):
    pass


class JSONError(TogetherException):
    pass


class InstanceError(TogetherException):
    def __init__(
        self,
        message: Optional[str] = None,
        http_body: Optional[str] = None,
        http_status: Optional[int] = None,
        json_body: Optional[Any] = None,
        headers: Optional[str] = None,
        model: Optional[str] = "model",
    ) -> None:
        message = f"No running instances for {model}. You can start an instance by navigating to the Together Playground at api.together.ai"
        super(InstanceError, self).__init__(
            message, http_body, http_status, json_body, headers
        )


class FileTypeError(TogetherException):
    pass


class AttributeError(TogetherException):
    pass



import re
import time


MEANINGLESS_WORDS = ["<pad>", "</s>", "<|endoftext|>"]
PRE_PROMPT = """\
Current Date: {}
Current Time: {}

"""


def clean_response(response: str) -> str:
    for word in MEANINGLESS_WORDS:
        response = response.replace(word, "")
    response = response.strip("\n")
    return response


class Conversation:
    def __init__(self, prompt_id: str, response_id: str) -> None:
        cur_date = time.strftime("%Y-%m-%d")
        cur_time = time.strftime("%H:%M:%S %p %Z")

        self._prompt_id = prompt_id
        self._response_id = response_id
        self._prompt = PRE_PROMPT.format(cur_date, cur_time)

    def push_context_turn(self, context: str) -> None:
        # for now, context is represented as a human turn
        self._prompt += f"{self._prompt_id}: {context}\n"

    def push_human_turn(self, query: str) -> None:
        self._prompt += f"{self._prompt_id}: {query}\n"
        self._prompt += f"{self._response_id}:"

    def push_model_response(self, response: str) -> None:
        # has_finished = self._prompt_id in response
        bot_turn = response.split(f"{self._prompt_id}:")[0]
        bot_turn = clean_response(bot_turn)
        # if it is truncated, then append "..." to the end of the response
        # if not has_finished:
        #    bot_turn += "..."

        self._prompt += f"{bot_turn}\n"

    def get_last_turn(self) -> str:
        human_tag = f"{self._prompt_id}:"
        bot_tag = f"{self._response_id}:"
        turns = re.split(f"({human_tag}|{bot_tag})\W?", self._prompt)
        return turns[-1]

    def get_raw_prompt(self) -> str:
        return self._prompt

    @classmethod
    def from_raw_prompt(cls, prompt_id: str, response_id: str, prompt: str):  # type: ignore
        convo = Conversation(prompt_id, response_id)
        convo._prompt = prompt
        return convo
